/********************************************************************
 Software License Agreement: 
 The software supplied herewith by Microchip Technology Incorporated
 (the "Company") for its PIC(R) Microcontroller is intended and
 supplied to you, the Company�s customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN �AS IS� CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
********************************************************************/

package com.hardkernel.android.ODROIDBluetoothLEDDemo;

import com.hardkernel.android.bluetooth.BluetoothMotorControlService;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Toast;

public class ODROIDBluetoothLEDDemo extends Activity {
	// Message types sent from the BluetoothChatService Handler
	public static final int MESSAGE_STATE_CHANGE = 1;
	public static final int MESSAGE_READ = 2;
	public static final int MESSAGE_DEVICE_NAME = 4;
	public static final int MESSAGE_TOAST = 5;
	
	// Key names received from the BluetoothChatService Handler
	public static final String DEVICE_NAME = "device_name";
	public static final String TOAST = "toast";
	
	// Intent request codes
	private static final int REQUEST_CONNECT_DEVICE = 1;
	private static final int REQUEST_ENABLE_BT = 2;
	
	// Buttons menu
	private final static int MENU_ITEM_CONNECT = Menu.FIRST;
	private final static int MENU_ITEM_DISCONNECT = Menu.FIRST + 1;
	private final static int MENU_ITEM_EXIT = Menu.FIRST + 2;
	
	// Name of the connected device
	private String mConnectedDeviceName = null;
	// Local Bluetooth adapter
	private BluetoothAdapter mBluetoothAdapter = null;
	// Member object for the chat services
	private BluetoothMotorControlService mCmdSendService = null;
	private boolean bluetoothPaired = false;
	
	private final static String TAG = "BluetoothRobot";

	public static final byte SEND_LETTER_GPIO_CONFIG			= 'g';
	public static final byte SEND_LETTER_GPIO_OUTPUT			= 'o';
	
	public static final int LED_1_PORT	= 2;
	public static final int LED_2_PORT	= 3;
	
	public static final int HIGH = 1;
	public static final int LOW = 0;
	
	private byte[] commandPacket = new byte[5];
	
	/** Called when the activity is first created. */
	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        setContentView(R.layout.main);
        
       	mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

		// If the adapter is null, then Bluetooth is not supported
		if (mBluetoothAdapter == null) {
			Toast.makeText(this, "Bluetooth is not available",
					Toast.LENGTH_SHORT).show();
			finish();
			return;
		}
        
        try {
        	LEDControl ledControl;
        	
			ledControl = ((LEDControl)findViewById(R.id.led_0));
			ledControl.setHandler(handler);
			ledControl.setPort(LED_1_PORT);
			
			ledControl = ((LEDControl)findViewById(R.id.led_1));
			ledControl.setHandler(handler);
			ledControl.setPort(LED_2_PORT);

        } catch (Exception e) {
        }
	}
	
	@Override
	public void onStart() {
		super.onStart();
		
		if (!mBluetoothAdapter.isEnabled()) {
			Intent enableIntent = new Intent(
					BluetoothAdapter.ACTION_REQUEST_ENABLE);
			startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
			// Otherwise, setup the chat session
		} else {
			if (mCmdSendService == null)
				setupBT();
		}
	}
	
    @Override
    public void onResume() {
    	super.onResume();
    	
    	// Performing this check in onResume() covers the case in which BT was
        // not enabled during onStart(), so we were paused to enable it...
        // onResume() will be called when ACTION_REQUEST_ENABLE activity
        // returns.
        if (mCmdSendService != null) {
            // Only if the state is STATE_NONE, do we know that we haven't
            // started already
            if (mCmdSendService.getState() == BluetoothMotorControlService.STATE_NONE) {
                // Start the Bluetooth chat services
                mCmdSendService.start();
            }
        }
    }
    
    @Override
    public void onPause() {
    	super.onPause();
    }
    
    @Override
	protected void onDestroy() {
		// TODO Auto-generated method stub
		super.onDestroy();
		// Stop the Bluetooth chat services
        if (mCmdSendService != null)
            mCmdSendService.stop();

	}

	@Override
	protected void onStop() {
		// TODO Auto-generated method stub
		super.onStop();
		// Stop the Bluetooth chat services
        if (mCmdSendService != null)
            mCmdSendService.stop();
	}

	private void setupBT() {
		// Initialize the BluetoothChatService to perform bluetooth connections
		mCmdSendService = new BluetoothMotorControlService(this, handler);
		mCmdSendService.setIndexOfMessages(MESSAGE_STATE_CHANGE, MESSAGE_READ, MESSAGE_DEVICE_NAME, MESSAGE_TOAST);
		mCmdSendService.setDeviceNameString(DEVICE_NAME);
		mCmdSendService.setToastString(TOAST);
	}
    
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		switch (requestCode) {
		case REQUEST_CONNECT_DEVICE:
			// When DeviceListActivity returns with a device to connect
			if (resultCode == Activity.RESULT_OK) {
				// Get the device MAC address
				String address = data.getExtras().getString(
						DeviceListActivity.EXTRA_DEVICE_ADDRESS);
				// Get the BLuetoothDevice object
				BluetoothDevice device = mBluetoothAdapter
						.getRemoteDevice(address);
				// Attempt to connect to the device
				mCmdSendService.connect(device);
			}
			break;
		case REQUEST_ENABLE_BT:
			// When the request to enable Bluetooth returns
			if (resultCode == Activity.RESULT_OK) {
				// Bluetooth is now enabled, so set up a chat session
				setupBT();
			} else {
				// User did not enable Bluetooth or an error occured
				Toast.makeText(this, R.string.bt_not_enabled_leaving,
						Toast.LENGTH_SHORT).show();
				finish();
			}
		}
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		menu.add(0, MENU_ITEM_CONNECT, 0, getString(R.string.connect)).setIcon(
				android.R.drawable.ic_menu_search);
		menu.add(0, MENU_ITEM_DISCONNECT, 0, getString(R.string.disconnect))
				.setIcon(android.R.drawable.ic_menu_close_clear_cancel);
		menu.add(0, MENU_ITEM_EXIT, 0, getString(R.string.exit)).setIcon(
				R.drawable.exit);
		return true;
	}
	
	@Override
	public boolean onMenuOpened(int featureId, Menu menu) {
		if (bluetoothPaired) {
			menu.findItem(MENU_ITEM_DISCONNECT).setVisible(true);
		} else {
			menu.findItem(MENU_ITEM_DISCONNECT).setVisible(false);
		}
		return super.onMenuOpened(featureId, menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case MENU_ITEM_CONNECT:
			// Launch the DeviceListActivity to see devices and do scan
			Intent serverIntent = new Intent(this, DeviceListActivity.class);
			startActivityForResult(serverIntent, REQUEST_CONNECT_DEVICE);
			return true;
		case MENU_ITEM_DISCONNECT:
			// Stop the Bluetooth chat services
			if (mCmdSendService != null)
				mCmdSendService.stop();
			return true;
		case MENU_ITEM_EXIT:
			finish();
			System.exit(0);
			return true;
		}
		return false;
	}
    
    /** 
     * Handler for receiving messages from the USB Manager thread or
     *   the LED control modules
     */
    private Handler handler = new Handler() {
    	@Override
    	public void handleMessage(Message msg) {
			
			switch(msg.what)
			{				
			case MESSAGE_STATE_CHANGE:
				switch (msg.arg1) {
				case BluetoothMotorControlService.STATE_CONNECTED:
					setTitle(getString(R.string.connected_to) + " " + mConnectedDeviceName);
					initDevice();
					bluetoothPaired = true;
					break;
				case BluetoothMotorControlService.STATE_CONNECTING:
					setTitle(R.string.title_connecting);
					break;
				case BluetoothMotorControlService.STATE_LISTEN:
				case BluetoothMotorControlService.STATE_NONE:
					setTitle(R.string.title_not_connected);
					bluetoothPaired = false;
					break;
				}
				break;
				
			case MESSAGE_DEVICE_NAME:
				// save the connected device's name
				mConnectedDeviceName = msg.getData().getString(DEVICE_NAME);
				Toast.makeText(
						getApplicationContext(),
						getString(R.string.connected_to) + mConnectedDeviceName,
						Toast.LENGTH_SHORT).show();
				break;
				
			case MESSAGE_TOAST:
				if (msg.getData().getString(TOAST).equals("unable connect")) {
					Toast.makeText(getApplicationContext(),
							getString(R.string.unable_connect),
							Toast.LENGTH_SHORT).show();
				} else if (msg.getData().getString(TOAST)
						.equals("connection lost")) {
					Toast.makeText(getApplicationContext(),
							getString(R.string.connection_lost),
							Toast.LENGTH_SHORT).show();
				} else {
					Toast.makeText(getApplicationContext(),
							msg.getData().getString(TOAST), Toast.LENGTH_SHORT)
							.show();
				}
				break;
				
			case SEND_LETTER_GPIO_CONFIG:
				if (mCmdSendService.getState() != BluetoothMotorControlService.STATE_CONNECTED) {
					Toast.makeText(getBaseContext(), R.string.not_connected, Toast.LENGTH_SHORT)
					.show();					
					return;
				}
				
				if (commandPacket[0] != 0) {
					return;
				}
				
				commandPacket[0] = SEND_LETTER_GPIO_CONFIG;
				commandPacket[1] = 0;
				commandPacket[2] = (byte) msg.arg1;
				commandPacket[3] = 0;
				commandPacket[4] = (byte) msg.arg2;

				mCmdSendService.write(commandPacket);
				break;
				
			case SEND_LETTER_GPIO_OUTPUT:
				if (mCmdSendService.getState() != BluetoothMotorControlService.STATE_CONNECTED) {
					Toast.makeText(getBaseContext(), R.string.not_connected, Toast.LENGTH_SHORT)
					.show();					
					return;
				}
				
				commandPacket[0] = SEND_LETTER_GPIO_OUTPUT;
				commandPacket[1] = 0;
				commandPacket[2] = (byte) msg.arg1;
				commandPacket[3] = 0;
				if (msg.arg2 == HIGH)
					commandPacket[4] = HIGH;
				else
					commandPacket[4] = LOW;
			
				mCmdSendService.write(commandPacket);
				break;
			
			case MESSAGE_READ:
				byte[] readBuf = (byte[]) msg.obj;	
				
				if (readBuf[0] == commandPacket[0] + ('A' - 'a')) {
					commandPacket[0] = 0;
				}
				
				switch(readBuf[0]) {
				}
			default:
				break;
			}//switch
    	} //handleMessage
    }; //handler
    
    private void initDevice() {
    	Log.e(TAG, "initDevice");
    	initGPIO(LED_1_PORT, true);
    	initGPIO(LED_2_PORT, true);
    }
    
    private void initGPIO(int port, boolean input) {    	
    	Log.e(TAG, "initGPIO(port:" + port + ")");
    	Message msg = Message.obtain(handler, SEND_LETTER_GPIO_CONFIG, port, input ? HIGH : LOW);
    	handler.sendMessage(msg);
    }
} //Class definition ODROIDBluetoothDemo
